@@ -1,4 +1,8 @@
 using System;
+using System.Globalization;
+using System.Linq;
+using System.Numerics;
+
 using NUnit.Framework;
 using Python.Runtime;
 
@@ -82,20 +86,10 @@ namespace Python.EmbeddingTest
             Assert.AreEqual(i, a.ToInt32());
         }
 
-        [Test]
-        public void TestCtorPtr()
-        {
-            var i = new PyInt(5);
-            Runtime.Runtime.XIncref(i.Handle);
-            var a = new PyInt(i.Handle);
-            Assert.AreEqual(5, a.ToInt32());
-        }
-
         [Test]
         public void TestCtorPyObject()
         {
             var i = new PyInt(5);
-            Runtime.Runtime.XIncref(i.Handle);
             var a = new PyInt(i);
             Assert.AreEqual(5, a.ToInt32());
         }
@@ -128,7 +122,7 @@ namespace Python.EmbeddingTest
 
             var ex = Assert.Throws<PythonException>(() => a = new PyInt(i));
 
-            StringAssert.StartsWith("ValueError : invalid literal for int", ex.Message);
+            StringAssert.StartsWith("invalid literal for int", ex.Message);
             Assert.IsNull(a);
         }
 
@@ -161,7 +155,7 @@ namespace Python.EmbeddingTest
             PyInt a = null;
 
             var ex = Assert.Throws<PythonException>(() => a = PyInt.AsInt(s));
-            StringAssert.StartsWith("ValueError : invalid literal for int", ex.Message);
+            StringAssert.StartsWith("invalid literal for int", ex.Message);
             Assert.IsNull(a);
         }
 
@@ -184,9 +178,47 @@ namespace Python.EmbeddingTest
         [Test]
         public void TestConvertToInt64()
         {
-            var a = new PyInt(5);
+            long val = 5 + (long)int.MaxValue;
+            var a = new PyInt(val);
             Assert.IsInstanceOf(typeof(long), a.ToInt64());
-            Assert.AreEqual(5, a.ToInt64());
+            Assert.AreEqual(val, a.ToInt64());
+        }
+
+        [Test]
+        public void ToBigInteger()
+        {
+            int[] simpleValues =
+            {
+                0, 1, 2,
+                0x10,
+                0x79,
+                0x80,
+                0x81,
+                0xFF,
+                0x123,
+                0x8000,
+                0x1234,
+                0x8001,
+                0x4000,
+                0xFF,
+            };
+            simpleValues = simpleValues.Concat(simpleValues.Select(v => -v)).ToArray();
+
+            var expected = simpleValues.Select(v => new BigInteger(v)).ToArray();
+            var actual = simpleValues.Select(v => new PyInt(v).ToBigInteger()).ToArray();
+
+            CollectionAssert.AreEqual(expected, actual);
+        }
+
+        [Test]
+        public void ToBigIntegerLarge()
+        {
+            BigInteger val = BigInteger.Pow(2, 1024) + 3;
+            var pyInt = new PyInt(val);
+            Assert.AreEqual(val, pyInt.ToBigInteger());
+            val = -val;
+            pyInt = new PyInt(val);
+            Assert.AreEqual(val, pyInt.ToBigInteger());
         }
     }
 }
