@@ -59,12 +59,12 @@ namespace Python.EmbeddingTest
         }
 
         [Test]
-        public void TestCtorPtr()
+        public void TestCtorBorrowed()
         {
             const string expected = "foo";
 
             var t = new PyString(expected);
-            var actual = new PyString(t.Handle);
+            var actual = new PyString(t.Reference);
 
             Assert.AreEqual(expected, actual.ToString());
         }
@@ -93,5 +93,24 @@ namespace Python.EmbeddingTest
             PyObject actual = new PyString(expected);
             Assert.AreEqual(expected, actual.ToString());
         }
+
+        [Test]
+        public void TestUnicodeSurrogateToString()
+        {
+            var expected = "foo\ud83d\udc3c";
+            var actual = PythonEngine.Eval("'foo\ud83d\udc3c'");
+            Assert.AreEqual(4, actual.Length());
+            Assert.AreEqual(expected, actual.ToString());
+        }
+
+        [Test]
+        public void TestUnicodeSurrogate()
+        {
+            const string expected = "foo\ud83d\udc3c"; // "foo🐼"
+            PyObject actual = new PyString(expected);
+            // python treats "foo🐼" as 4 characters, dotnet as 5
+            Assert.AreEqual(4, actual.Length());
+            Assert.AreEqual(expected, actual.ToString());
+        }
     }
 }
