@@ -13,40 +13,26 @@ Requirements:
     - clang
 """
 
-from __future__ import print_function
-
-import logging
 import os
+import shutil
 import sys
 import sysconfig
 import subprocess
 
-if sys.version_info.major > 2:
-    from io import StringIO
-else:
-    from StringIO import StringIO
-
+from io import StringIO
+from pathlib import Path
 from pycparser import c_ast, c_parser
 
-_log = logging.getLogger()
-logging.basicConfig(level=logging.DEBUG)
-
-PY_MAJOR = sys.version_info[0]
-PY_MINOR = sys.version_info[1]
-
 # rename some members from their C name when generating the C#
 _typeoffset_member_renames = {
     "ht_name": "name",
-    "ht_qualname": "qualname"
+    "ht_qualname": "qualname",
+    "getitem": "spec_cache_getitem",
 }
 
 
 def _check_output(*args, **kwargs):
-    """Check output wrapper for py2/py3 compatibility"""
-    output = subprocess.check_output(*args, **kwargs)
-    if PY_MAJOR == 2:
-        return output
-    return output.decode("ascii")
+    return subprocess.check_output(*args, **kwargs, encoding="utf8")
 
 
 class AstParser(object):
@@ -92,7 +78,7 @@ class AstParser(object):
             self.visit_identifier(node)
 
     def visit_ast(self, ast):
-        for name, node in ast.children():
+        for _name, node in ast.children():
             self.visit(node)
 
     def visit_typedef(self, typedef):
@@ -113,7 +99,7 @@ class AstParser(object):
                 self.visit(decl)
                 self._struct_members_stack.pop(0)
             self._struct_stack.pop(0)
-        elif self._ptr_decl_depth:
+        elif self._ptr_decl_depth or self._struct_members_stack:
             # the struct is empty, but add it as a member to the current
             # struct as the current member maybe a pointer to it.
             self._add_struct_member(struct.name)
@@ -141,7 +127,8 @@ class AstParser(object):
         current_struct = self._struct_stack[0]
         member_name = self._struct_members_stack[0]
         struct_members = self._struct_members.setdefault(
-            self._get_struct_name(current_struct), [])
+            self._get_struct_name(current_struct), []
+        )
 
         # get the node associated with this type
         node = None
@@ -179,7 +166,6 @@ class AstParser(object):
 
 
 class Writer(object):
-
     def __init__(self):
         self._stream = StringIO()
 
@@ -193,45 +179,56 @@ class Writer(object):
         return self._stream.getvalue()
 
 
-def preprocess_python_headers():
+def preprocess_python_headers(*, cc=None, include_py=None):
     """Return Python.h pre-processed, ready for parsing.
     Requires clang.
     """
-    fake_libc_include = os.path.join(os.path.dirname(__file__),
-                                     "fake_libc_include")
+    this_path = Path(__file__).parent
+
+    fake_libc_include = this_path / "fake_libc_include"
     include_dirs = [fake_libc_include]
 
-    include_py = sysconfig.get_config_var("INCLUDEPY")
+    if cc is None:
+        cc = shutil.which("clang")
+    if cc is None:
+        cc = shutil.which("gcc")
+    if cc is None:
+        raise RuntimeError("No suitable C compiler found, need clang or gcc")
+
+    if include_py is None:
+        include_py = sysconfig.get_config_var("INCLUDEPY")
+    include_py = Path(include_py)
+
     include_dirs.append(include_py)
 
-    include_args = [c for p in include_dirs for c in ["-I", p]]
+    include_args = [c for p in include_dirs for c in ["-I", str(p)]]
 
+    # fmt: off
     defines = [
         "-D", "__attribute__(x)=",
         "-D", "__inline__=inline",
         "-D", "__asm__=;#pragma asm",
         "-D", "__int64=long long",
-        "-D", "_POSIX_THREADS"
+        "-D", "_POSIX_THREADS",
     ]
 
-    if os.name == 'nt':
+    if sys.platform == "win32":
         defines.extend([
             "-D", "__inline=inline",
             "-D", "__ptr32=",
             "-D", "__ptr64=",
             "-D", "__declspec(x)=",
         ])
+    #fmt: on
 
     if hasattr(sys, "abiflags"):
         if "d" in sys.abiflags:
             defines.extend(("-D", "PYTHON_WITH_PYDEBUG"))
-        if "m" in sys.abiflags:
-            defines.extend(("-D", "PYTHON_WITH_PYMALLOC"))
         if "u" in sys.abiflags:
             defines.extend(("-D", "PYTHON_WITH_WIDE_UNICODE"))
 
-    python_h = os.path.join(include_py, "Python.h")
-    cmd = ["clang", "-pthread"] + include_args + defines + ["-E", python_h]
+    python_h = include_py / "Python.h"
+    cmd = [cc, "-pthread"] + include_args + defines + ["-E", str(python_h)]
 
     # normalize as the parser doesn't like windows line endings.
     lines = []
@@ -242,85 +239,56 @@ def preprocess_python_headers():
     return "\n".join(lines)
 
 
-
-def gen_interop_head(writer):
-    defines = [
-        "PYTHON{0}{1}".format(PY_MAJOR, PY_MINOR)
-    ]
-
-    if hasattr(sys, "abiflags"):
-        if "d" in sys.abiflags:
-            defines.append("PYTHON_WITH_PYDEBUG")
-        if "m" in sys.abiflags:
-            defines.append("PYTHON_WITH_PYMALLOC")
-        if "u" in sys.abiflags:
-            defines.append("PYTHON_WITH_WIDE_UNICODE")
-
+def gen_interop_head(writer, version, abi_flags):
     filename = os.path.basename(__file__)
-    defines_str = " && ".join(defines)
-    class_definition = """
-// Auto-generated by %s.
+    class_definition = f"""
+// Auto-generated by {filename}.
 // DO NOT MODIFY BY HAND.
 
+// Python {".".join(version[:2])}: ABI flags: '{abi_flags}'
+
+// ReSharper disable InconsistentNaming
+// ReSharper disable IdentifierTypo
 
-#if %s
 using System;
-using System.Collections;
-using System.Collections.Specialized;
+using System.Diagnostics.CodeAnalysis;
 using System.Runtime.InteropServices;
-using System.Reflection;
-using System.Text;
+
+using Python.Runtime.Native;
 
 namespace Python.Runtime
-{
-""" % (filename, defines_str)
+{{"""
     writer.extend(class_definition)
 
 
 def gen_interop_tail(writer):
     tail = """}
-#endif
 """
     writer.extend(tail)
 
 
-def gen_heap_type_members(parser, writer):
+def gen_heap_type_members(parser, writer, type_name):
     """Generate the TypeOffset C# class"""
     members = parser.get_struct_members("PyHeapTypeObject")
-    class_definition = """
+    class_definition = f"""
+    [SuppressMessage("Style", "IDE1006:Naming Styles",
+                     Justification = "Following CPython",
+                     Scope = "type")]
+
     [StructLayout(LayoutKind.Sequential)]
-    internal static class TypeOffset
-    {
-        static TypeOffset()
-        {
-            Type type = typeof(TypeOffset);
-            FieldInfo[] fi = type.GetFields();
-            int size = IntPtr.Size;
-            for (int i = 0; i < fi.Length; i++)
-            {
-                fi[i].SetValue(null, i * size);
-            }
-        }
-
-        public static int magic()
-        {
-            return ob_size;
-        }
-
+    internal class {type_name} : GeneratedTypeOffsets, ITypeOffsets
+    {{
+        public {type_name}() {{ }}
         // Auto-generated from PyHeapTypeObject in Python.h
 """
 
     # All the members are sizeof(void*) so we don't need to do any
     # extra work to determine the size based on the type.
-    for name, tpy in members:
+    for name, _type in members:
         name = _typeoffset_member_renames.get(name, name)
-        class_definition += "        public static int %s = 0;\n" % name
-
-    class_definition += """
-        /* here are optional user slots, followed by the members. */
-        public static int members = 0;
-    }
+        class_definition += "        public int %s  { get; private set; }\n" % name
 
+    class_definition += """    }
 """
     writer.extend(class_definition)
 
@@ -331,30 +299,18 @@ def gen_structure_code(parser, writer, type_name, indent):
         return False
     out = writer.append
     out(indent, "[StructLayout(LayoutKind.Sequential)]")
-    out(indent, "internal struct %s" % type_name)
+    out(indent, f"internal struct {type_name}")
     out(indent, "{")
-    for name, tpy in members:
-        out(indent + 1, "public IntPtr %s;" % name)
+    for name, _type in members:
+        out(indent + 1, f"public IntPtr {name};")
     out(indent, "}")
     out()
     return True
 
 
-def gen_supported_slot_record(writer, types, indent):
-    out = writer.append
-    out(indent, "internal static partial class SlotTypes")
-    out(indent, "{")
-    out(indent + 1, "public static readonly Type[] Types = {")
-    for name in types:
-        out(indent + 2, "typeof(%s)," % name)
-    out(indent + 1, "};")
-    out(indent, "}")
-    out()
-
-
-def main():
+def main(*, cc=None, include_py=None, version=None, out=None):
     # preprocess Python.h and build the AST
-    python_h = preprocess_python_headers()
+    python_h = preprocess_python_headers(cc=cc, include_py=include_py)
     parser = c_parser.CParser()
     ast = parser.parse(python_h)
 
@@ -363,34 +319,47 @@ def main():
     ast_parser.visit(ast)
 
     writer = Writer()
-    # generate the C# code
-    gen_interop_head(writer)
 
-    gen_heap_type_members(ast_parser, writer)
-    slots_types = [
-        "PyNumberMethods",
-        "PySequenceMethods",
-        "PyMappingMethods",
-        "PyAsyncMethods",
-        "PyBufferProcs",
-    ]
-    supported_types = []
-    indent = 1
-    for type_name in slots_types:
-        if not gen_structure_code(ast_parser, writer, type_name, indent):
-            continue
-        supported_types.append(type_name)
-    gen_supported_slot_record(writer, supported_types, indent)
+    if include_py and not version:
+        raise RuntimeError("If the include path is overridden, version must be "
+                "defined"
+                )
+
+    if version:
+        version = version.split('.')
+    else:
+        version = sys.version_info
+
+    # generate the C# code
+    abi_flags = getattr(sys, "abiflags", "").replace("m", "")
+    gen_interop_head(writer, version, abi_flags)
+
+    type_name = f"TypeOffset{version[0]}{version[1]}{abi_flags}"
+    gen_heap_type_members(ast_parser, writer, type_name)
 
     gen_interop_tail(writer)
 
     interop_cs = writer.to_string()
-    if len(sys.argv) > 1:
-        with open(sys.argv[1], "w") as fh:
-            fh.write(interop_cs)
-    else:
+    if not out or out == "-":
         print(interop_cs)
+    else:
+        with open(out, "w") as fh:
+            fh.write(interop_cs)
 
 
 if __name__ == "__main__":
-    sys.exit(main())
+    import argparse
+
+    a = argparse.ArgumentParser("Interop file generator for Python.NET")
+    a.add_argument("--cc", help="C compiler to use, either clang or gcc")
+    a.add_argument("--include-py", help="Include path of Python")
+    a.add_argument("--version", help="Python version")
+    a.add_argument("--out", help="Output path", default="-")
+    args = a.parse_args()
+
+    sys.exit(main(
+        cc=args.cc,
+        include_py=args.include_py,
+        out=args.out,
+        version=args.version
+        ))
